`timescale 1ns / 1ps

module lc4_divider_one_iter(dividend_in, divisor_in, remainder_in, quotient_in, 
                            dividend_out, remainder_out, quotient_out);
   
   input [15:0] dividend_in, divisor_in, remainder_in, quotient_in;
   output [15:0] dividend_out, remainder_out, quotient_out;
   
   
   wire [15:0]   remainder = {remainder_in[14:0],dividend_in[15]};
   wire          condition = (remainder < divisor_in);
   
   assign quotient_out = condition ? {quotient_in[14:0], 1'b0} : {quotient_in[14:0], 1'b1};
   assign remainder_out = condition ? remainder : remainder - divisor_in;
   assign dividend_out = {dividend_in[14:0], 1'b0};
   
endmodule

module lc4_divider_four_iter(dividend_in, divisor_in, remainder_in, quotient_in, 
                             dividend_out, remainder_out, quotient_out);
   
   input [15:0] dividend_in, divisor_in, remainder_in, quotient_in;
   output [15:0] dividend_out, remainder_out, quotient_out;

   wire [15:0]   iter1_dividend_out, iter1_remainder_out, iter1_quotient_out;
   wire [15:0]   iter2_dividend_out, iter2_remainder_out, iter2_quotient_out;
   wire [15:0]   iter3_dividend_out, iter3_remainder_out, iter3_quotient_out;
   wire [15:0]   iter4_dividend_out, iter4_remainder_out, iter4_quotient_out;

   wire [15:0]   iter1_dividend_in = dividend_in;
   wire [15:0]   iter2_dividend_in = iter1_dividend_out;
   wire [15:0]   iter3_dividend_in = iter2_dividend_out;
   wire [15:0]   iter4_dividend_in = iter3_dividend_out;

   wire [15:0]   iter1_remainder_in = remainder_in;
   wire [15:0]   iter2_remainder_in = iter1_remainder_out;
   wire [15:0]   iter3_remainder_in = iter2_remainder_out;
   wire [15:0]   iter4_remainder_in = iter3_remainder_out;
   
   wire [15:0]   iter1_quotient_in = quotient_in;
   wire [15:0]   iter2_quotient_in = iter1_quotient_out;
   wire [15:0]   iter3_quotient_in = iter2_quotient_out;
   wire [15:0]   iter4_quotient_in = iter3_quotient_out;

   
   lc4_divider_one_iter iter1 (iter1_dividend_in, divisor_in, iter1_remainder_in, iter1_quotient_in, 
                               iter1_dividend_out, iter1_remainder_out, iter1_quotient_out);
   
   lc4_divider_one_iter iter2 (iter2_dividend_in, divisor_in, iter2_remainder_in, iter2_quotient_in, 
                               iter2_dividend_out, iter2_remainder_out, iter2_quotient_out);
   
   lc4_divider_one_iter iter3 (iter3_dividend_in, divisor_in, iter3_remainder_in, iter3_quotient_in, 
                               iter3_dividend_out, iter3_remainder_out, iter3_quotient_out);
   
   lc4_divider_one_iter iter4 (iter4_dividend_in, divisor_in, iter4_remainder_in, iter4_quotient_in, 
                               iter4_dividend_out, iter4_remainder_out, iter4_quotient_out);
   
   assign dividend_out = iter4_dividend_out;
   assign remainder_out = iter4_remainder_out;
   assign quotient_out = iter4_quotient_out;

endmodule

